package io.papermc.paper.configuration.serializer;

import io.leangen.geantyref.TypeToken;
import io.papermc.paper.util.ObfHelper;
import java.lang.reflect.Type;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import net.minecraft.network.protocol.Packet;
import org.checkerframework.checker.nullness.qual.Nullable;
import org.spongepowered.configurate.serialize.ScalarSerializer;
import org.spongepowered.configurate.serialize.SerializationException;

@SuppressWarnings("Convert2Diamond")
public final class PacketClassSerializer extends ScalarSerializer<Class<? extends Packet<?>>> {
    private static final TypeToken<Class<? extends Packet<?>>> TYPE = new TypeToken<Class<? extends Packet<?>>>() {};
    private static final List<String> SUBPACKAGES = List.of("game", "handshake", "login", "status");
    private static final Map<String, String> MOJANG_TO_OBF = Optional.ofNullable(ObfHelper.INSTANCE.mappingsByObfName())
        .map(Map::entrySet)
        .map(Collection::stream)
        .map(stream -> stream.collect(Collectors.toMap(entry -> entry.getValue().mojangName(), entry -> entry.getValue().obfName())))
        .orElseGet(Collections::emptyMap);

    public PacketClassSerializer() {
        super(TYPE);
    }

    @SuppressWarnings("unchecked")
    @Override
    public Class<? extends Packet<?>> deserialize(final Type type, final Object obj) throws SerializationException {
        @Nullable Class<?> packetClass = null;
        for (final String subpackage : SUBPACKAGES) {
            final String fullClassName = "net.minecraft.network.protocol." + subpackage + "." + obj;
            try {
                packetClass = Class.forName(fullClassName);
                break;
            } catch (final ClassNotFoundException ex) {
                final @Nullable String spigotClassName = MOJANG_TO_OBF.get(fullClassName);
                if (spigotClassName != null) {
                    try {
                        packetClass = Class.forName(spigotClassName);
                    } catch (final ClassNotFoundException ignore) {}
                }
            }
        }
        if (packetClass == null || !Packet.class.isAssignableFrom(packetClass)) {
            throw new SerializationException("Could not deserialize a packet from " + obj);
        }
        return (Class<? extends Packet<?>>) packetClass;
    }

    @Override
    protected Object serialize(final Class<? extends Packet<?>> item, final Predicate<Class<?>> typeSupported) {
        //TODO always serialize the mapped class name to not break on switching between mapped/unmapped servers
        return item.getSimpleName();
    }
}
